// SPDX-License-Identifier: MIT
//
// Copyright IBM Corp. 2023

use crate::cli::ListSecretOpt;
use anyhow::{Context, Result};
use log::warn;
use pv::{
    misc::{get_writer_from_cli_file_arg, STDOUT},
    uv::UvDevice,
    uv::{ListCmd, SecretList},
};

/// Do a List Secrets UVC
pub fn list(opt: &ListSecretOpt) -> Result<()> {
    let uv = UvDevice::open()?;
    let mut cmd = ListCmd::default();
    uv.send_cmd(&mut cmd)?;

    let secret_list: SecretList = cmd.try_into()?;
    let mut wr_out = get_writer_from_cli_file_arg(&opt.output)?;

    // write list to file
    match (&opt.bin, &opt.yaml) {
        (true, false) => secret_list
            .encode(&mut wr_out)
            .context("Cannot encode secret list")?,
        (false, true) => write!(wr_out, "{}", serde_yaml::to_string(&secret_list)?)
            .context("Cannot generate yaml output")?,
        (false, false) => write!(wr_out, "{secret_list}").context("Cannot generate output")?,
        _ => unreachable!("--bin and --yaml both set (CLI parser bug)"),
    }
    wr_out.flush()?;

    if opt.output != STDOUT {
        warn!(
            "Successfully wrote the list of secrets to '{}'",
            &opt.output
        );
    }
    Ok(())
}
